@@ -495,10 +495,12 @@ arm_need_rel_to_rela (DSO *dso, int first, int last)
 }
 
 static int
-arm_arch_prelink (DSO *dso)
+arm_arch_prelink (struct prelink_info *info)
 {
+  DSO *dso;
   int i;
 
+  dso = info->dso;
   if (dso->info[DT_PLTGOT])
     {
       /* Write address of .plt into got[1].
