@@ -309,10 +309,12 @@ cris_need_rel_to_rela (DSO *dso, int first, int last)
 }
 
 static int
-cris_arch_prelink (DSO *dso)
+cris_arch_prelink (struct prelink_info *info)
 {
+  DSO *dso;
   int i;
 
+  dso = info->dso;
   if (dso->info[DT_PLTGOT])
     {
       /* Write address of .plt + 28 into got[1].
