@@ -599,10 +599,12 @@ i386_need_rel_to_rela (DSO *dso, int first, int last)
 }
 
 static int
-i386_arch_prelink (DSO *dso)
+i386_arch_prelink (struct prelink_info *info)
 {
+  DSO *dso;
   int i;
 
+  dso = info->dso;
   if (dso->info[DT_PLTGOT])
     {
       /* Write address of .plt + 0x16 into got[1].
