@@ -471,7 +471,7 @@ sparc_need_rel_to_rela (DSO *dso, int first, int last)
 }
 
 static int
-sparc_arch_prelink (DSO *dso)
+sparc_arch_prelink (struct prelink_info *info)
 {
   return 0;
 }
