@@ -350,10 +350,12 @@ x86_64_need_rel_to_rela (DSO *dso, int first, int last)
 }
 
 static int
-x86_64_arch_prelink (DSO *dso)
+x86_64_arch_prelink (struct prelink_info *info)
 {
+  DSO *dso;
   int i;
 
+  dso = info->dso;
   if (dso->info[DT_PLTGOT])
     {
       /* Write address of .plt + 0x16 into got[1].
