@@ -107,6 +107,15 @@ read_dynamic (DSO *dso)
 		    dso->info_DT_GNU_HASH = dyn.d_un.d_val;
 		    dso->info_set_mask |= (1ULL << DT_GNU_HASH_BIT);
 		  }
+		if (dso->ehdr.e_machine == EM_MIPS)
+		  {
+		    if (dyn.d_tag == DT_MIPS_LOCAL_GOTNO)
+		      dso->info_DT_MIPS_LOCAL_GOTNO = dyn.d_un.d_val;
+		    else if (dyn.d_tag == DT_MIPS_GOTSYM)
+		      dso->info_DT_MIPS_GOTSYM = dyn.d_un.d_val;
+		    else if (dyn.d_tag == DT_MIPS_SYMTABNO)
+		      dso->info_DT_MIPS_SYMTABNO = dyn.d_un.d_val;
+		  }
 	      }
 	    if (ndx < maxndx)
 	      break;
@@ -1011,6 +1020,38 @@ error_out:
   return 1;
 }
 
+/* Return true if the value of symbol SYM, which belongs to DSO,
+   should be treated as an address within the DSO, and should
+   therefore track DSO's relocations.  */
+
+int
+adjust_symbol_p (DSO *dso, GElf_Sym *sym)
+{
+  if (sym->st_shndx == SHN_ABS
+      && sym->st_value != 0
+      && GELF_ST_TYPE (sym->st_info) <= STT_FUNC)
+    /* This is problematic.  How do we find out if
+       we should relocate this?  Assume we should.  */
+    return 1;
+
+  /* If a MIPS object does not define a symbol, but has a lazy binding
+     stub for it, st_value will point to that stub.  Note that unlike
+     other targets, these stub addresses never participate in symbol
+     lookup; the stubs can only be called by the object that defines them.
+     st_values are only used in this way so that the associated GOT entry
+     can store a Quickstart value without losing the original stub
+     address.  */
+  if (dso->ehdr.e_machine == EM_MIPS
+      && sym->st_shndx == SHN_UNDEF
+      && sym->st_value != 0)
+    return 1;
+
+  return (sym->st_shndx > SHN_UNDEF
+	  && sym->st_shndx < dso->ehdr.e_shnum
+	  && ELF32_ST_TYPE (sym->st_info) != STT_TLS
+	  && RELOCATE_SCN (dso->shdr[sym->st_shndx].sh_flags));
+}
+
 static int
 adjust_symtab (DSO *dso, int n, GElf_Addr start, GElf_Addr adjust)
 {
@@ -1025,29 +1066,7 @@ adjust_symtab (DSO *dso, int n, GElf_Addr start, GElf_Addr adjust)
       for (ndx = 0; ndx < maxndx; ++ndx)
 	{
 	  gelfx_getsym (dso->elf, data, ndx, &sym);
-	  if (sym.st_shndx == SHN_ABS && sym.st_value != 0
-	      && GELF_ST_TYPE (sym.st_info) <= STT_FUNC)
-	    {
-	      /* This is problematic.  How do we find out if
-		 we should relocate this?  Assume we should.  */
-	      if (sym.st_value >= start)
-		{
-		  sym.st_value += adjust;
-		  sym.st_value &= dso->mask;
-		  gelfx_update_sym (dso->elf, data, ndx, &sym);
-		}
-	      continue;
-	    }
-
-	  if (sym.st_shndx <= SHN_UNDEF
-	      || sym.st_shndx >= dso->ehdr.e_shnum
-	      || ELF32_ST_TYPE (sym.st_info) == STT_TLS)
-	    continue;
-
-	  if (! RELOCATE_SCN (dso->shdr[sym.st_shndx].sh_flags))
-	    continue;
-
-	  if (sym.st_value >= start)
+	  if (adjust_symbol_p (dso, &sym) && sym.st_value >= start)
 	    {
 	      sym.st_value += adjust;
 	      gelfx_update_sym (dso->elf, data, ndx, &sym);
@@ -1295,6 +1314,10 @@ adjust_dso (DSO *dso, GElf_Addr start, GElf_Addr adjust)
 {
   int i;
 
+  if (dso->arch->arch_adjust
+      && dso->arch->arch_adjust (dso, start, adjust))
+    return 1;
+
   if (dso->ehdr.e_entry >= start)
     {
       dso->ehdr.e_entry += adjust;
