@@ -1,4 +1,4 @@
-/* Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006 Red Hat, Inc.
+/* Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007 Red Hat, Inc.
    Written by Jakub Jelinek <jakub@redhat.com>, 2001.
 
    This program is free software; you can redistribute it and/or modify
@@ -389,6 +389,7 @@ struct prelink_symbol
 struct prelink_conflict
 {
   struct prelink_conflict *next;
+  struct prelink_conflict *next2;
   /* Object which it was relocated to.  */
   union
     {
@@ -407,6 +408,14 @@ struct prelink_conflict
   int used;
 };
 
+struct prelink_conflicts
+{
+  struct prelink_conflict *first;
+  struct prelink_conflict **hash;
+  struct prelink_conflict **hash2;
+  size_t count;
+};
+
 #define conflict_lookup_value(cfl)					  \
   (((cfl)->reloc_class != RTYPE_CLASS_TLS ? (cfl)->lookup.ent->base : 0)  \
    + (cfl)->lookupval)
@@ -417,8 +426,8 @@ struct prelink_info
   DSO **dsos;
   struct prelink_entry *ent;
   struct prelink_symbol *symbols;
-  struct prelink_conflict **conflicts;
-  struct prelink_conflict *curconflicts;
+  struct prelink_conflicts *conflicts;
+  struct prelink_conflicts *curconflicts;
   struct prelink_tls *tls, *curtls;
   const char **sonames;
   char *dynbss, *sdynbss;
@@ -461,10 +470,11 @@ int prelink_undo (DSO *dso);
 int prelink_verify (const char *filename);
 
 int gather_object (const char *dir, int deref, int onefs);
-int gather_config (const char *config);
+int read_config (const char *config);
+int gather_config (void);
 int gather_check_libs (void);
 int add_to_blacklist (const char *name, int deref, int onefs);
-int blacklist_from_config (const char *config);
+int blacklist_from_config (void);
 
 FILE *execve_open (const char *path, char *const argv[], char *const envp[]);
 int execve_close (FILE *f);
